@@ -1,73 +1,69 @@
 package List::Compare::Base::_Auxiliary;
-$VERSION = 0.39;
+$VERSION = 0.53;
 use Carp;
 @ISA = qw(Exporter);
 @EXPORT_OK = qw|
     _validate_2_seenhashes
     _validate_seen_hash
     _validate_multiple_seenhashes
-    _calculate_union_xintersection_only
-    _calculate_seen_xintersection_only
+    _calculate_array_seen_only
     _calculate_seen_only
-    _calculate_xintersection_only
+    _calculate_intermediate
     _calculate_union_only
     _calculate_union_seen_only
-    _calculate_hash_intersection
-    _calculate_hash_shared
+    _calculate_sharedref
     _subset_subengine
-    _chart_engine_regular 
+    _chart_engine_regular
     _chart_engine_multiple
     _equivalent_subengine
     _index_message1
     _index_message2
     _index_message3
     _index_message4
-    _prepare_listrefs 
-    _subset_engine_multaccel 
+    _prepare_listrefs
+    _subset_engine_multaccel
     _calc_seen
     _calc_seen1
-    _equiv_engine 
-    _argument_checker_0 
-    _argument_checker 
-    _argument_checker_1 
-    _argument_checker_2 
-    _argument_checker_3 
-    _argument_checker_3a 
+    _equiv_engine
+    _argument_checker_0
+    _argument_checker
+    _argument_checker_1
+    _argument_checker_2
+    _argument_checker_3
+    _argument_checker_3a
     _argument_checker_4
-    _alt_construct_tester 
-    _alt_construct_tester_1 
-    _alt_construct_tester_2 
-    _alt_construct_tester_3 
-    _alt_construct_tester_4 
-    _alt_construct_tester_5 
+    _alt_construct_tester
+    _alt_construct_tester_1
+    _alt_construct_tester_2
+    _alt_construct_tester_3
+    _alt_construct_tester_4
+    _alt_construct_tester_5
 |;
 %EXPORT_TAGS = (
     calculate => [ qw(
-        _calculate_union_xintersection_only
-        _calculate_seen_xintersection_only
+        _calculate_array_seen_only
         _calculate_seen_only
-        _calculate_xintersection_only
+        _calculate_intermediate
         _calculate_union_only
         _calculate_union_seen_only
-        _calculate_hash_intersection
-        _calculate_hash_shared
+        _calculate_sharedref
     ) ],
     checker => [ qw(
-        _argument_checker_0 
-        _argument_checker 
-        _argument_checker_1 
-        _argument_checker_2 
-        _argument_checker_3 
-        _argument_checker_3a 
+        _argument_checker_0
+        _argument_checker
+        _argument_checker_1
+        _argument_checker_2
+        _argument_checker_3
+        _argument_checker_3a
         _argument_checker_4
     ) ],
     tester => [ qw(
-        _alt_construct_tester 
-        _alt_construct_tester_1 
-        _alt_construct_tester_2 
-        _alt_construct_tester_3 
-        _alt_construct_tester_4 
-        _alt_construct_tester_5 
+        _alt_construct_tester
+        _alt_construct_tester_1
+        _alt_construct_tester_2
+        _alt_construct_tester_3
+        _alt_construct_tester_4
+        _alt_construct_tester_5
     ) ],
 );
 use strict;
@@ -85,7 +81,7 @@ sub _validate_2_seenhashes {
         } else {
             $badentriesL{$_} = ${$refL}{$_};
         }
-    } 
+    }
     foreach (keys %$refR) {
         if (${$refR}{$_} =~ /^\d+$/ and ${$refR}{$_} > 0) {
             $seenR{$_} = ${$refR}{$_};
@@ -99,12 +95,12 @@ sub _validate_2_seenhashes {
         $msg .= "  These elements have invalid values:\n";
         if (keys %badentriesL) {
             $msg .= "  First hash in arguments:\n";
-            $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n" 
+            $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n"
                 foreach (sort keys %badentriesL);
-        } 
+        }
         if (keys %badentriesR) {
             $msg .= "  Second hash in arguments:\n";
-            $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n" 
+            $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n"
                 foreach (sort keys %badentriesR);
         }
         $msg .= "Correct invalid values before proceeding";
@@ -116,15 +112,15 @@ sub _validate_2_seenhashes {
 sub _validate_seen_hash {
     if (@_ > 2) {
         _validate_multiple_seenhashes( [@_] );
-    } else { 
+    } else {
         my ($l, $r) = @_;
         my (%badentriesL, %badentriesR);
         foreach (keys %$l) {
-            $badentriesL{$_} = ${$l}{$_} 
+            $badentriesL{$_} = ${$l}{$_}
                 unless (${$l}{$_} =~ /^\d+$/ and ${$l}{$_} > 0);
-        } 
+        }
         foreach (keys %$r) {
-            $badentriesR{$_} = ${$r}{$_} 
+            $badentriesR{$_} = ${$r}{$_}
                 unless (${$r}{$_} =~ /^\d+$/ and ${$r}{$_} > 0);
         }
         my $msg = q{};
@@ -133,12 +129,12 @@ sub _validate_seen_hash {
             $msg .= "  These elements have invalid values:\n";
             if (keys %badentriesL) {
                 $msg .= "  First hash in arguments:\n";
-                $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n" 
+                $msg .= "     Key:  $_\tValue:  $badentriesL{$_}\n"
                     foreach (sort keys %badentriesL);
-            } 
+            }
             if (keys %badentriesR) {
                 $msg .= "  Second hash in arguments:\n";
-                $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n" 
+                $msg .= "     Key:  $_\tValue:  $badentriesR{$_}\n"
                     foreach (sort keys %badentriesR);
             }
             $msg .= "Correct invalid values before proceeding";
@@ -149,26 +145,22 @@ sub _validate_seen_hash {
 
 sub _validate_multiple_seenhashes {
     my $hashrefsref = shift;
-    my @hashrefs = @{$hashrefsref};
-    my (%badentries, $badentriesflag);
-    for (my $i = 0; $i <= $#hashrefs; $i++) {
-        my %seenhash = %{$hashrefs[$i]};
-        foreach (keys %seenhash) {
-            unless ($seenhash{$_} =~ /^\d+$/ and $seenhash{$_} > 0) {
-                $badentries{$i}{$_} = $seenhash{$_};
-                $badentriesflag++;
+    my (%badentries);
+    for (my $i = 0; $i <= $#{$hashrefsref}; $i++) {
+        foreach my $k (keys %{$hashrefsref->[$i]}) {
+            unless ($hashrefsref->[$i]->{$k} =~ /^\d+$/ and $hashrefsref->[$i]->{$k} > 0) {
+                $badentries{$i}{$k} = $hashrefsref->[$i]->{$k};
             }
         }
     }
     my $msg = q{};
-    if ($badentriesflag) {
+    if (scalar(keys %badentries)) {
         $msg .= "\nValues in a 'seen-hash' must be positive integers.\n";
         $msg .= "  These elements have invalid values:\n\n";
-        foreach (sort keys %badentries) {
-            $msg .= "    Hash $_:\n";
-            my %pairs = %{$badentries{$_}};
-            foreach my $val (sort keys %pairs) {
-                $msg .= "        Bad key-value pair:  $val\t$pairs{$val}\n";
+        foreach my $b (sort keys %badentries) {
+            $msg .= "    Hash $b:\n";
+            foreach my $val (sort keys %{$badentries{$b}}) {
+                $msg .= "        Bad key-value pair:  $val\t$badentries{$b}->{$val}\n";
             }
         }
         $msg .= "Correct invalid values before proceeding";
@@ -185,49 +177,17 @@ sub _list_builder {
     }
 }
 
-sub _calculate_union_xintersection_only {
+sub _calculate_array_seen_only {
     my $aref = shift;
-    my (%union, %xintersection);
+    my (@seen);
     for (my $i = 0; $i <= $#{$aref}; $i++) {
         my %seenthis = ();
-        foreach my $h ( _list_builder($aref, $i) ) {
-            $seenthis{$h}++;
-            $union{$h}++;
-        }
-        for (my $j = $i+1; $j <=$#{$aref}; $j++) {
-            my %seenthat = ();
-            my %seenintersect = ();
-            my $ilabel = $i . '_' . $j;
-            $seenthat{$_}++ foreach ( _list_builder($aref, $j) );
-            foreach my $k (keys %seenthat) {
-                $seenintersect{$k}++ if (exists $seenthis{$k});
-            }
-            $xintersection{$ilabel} = \%seenintersect;
+        foreach my $el ( _list_builder($aref, $i) ) {
+            $seenthis{$el}++;
         }
+        push @seen, \%seenthis;
     }
-    return (\%union, \%xintersection);
-}
-
-sub _calculate_seen_xintersection_only {
-    my $aref = shift;
-    my (%xintersection, %seen);
-    for (my $i = 0; $i <= $#{$aref}; $i++) {
-        my %seenthis = ();
-        foreach my $h ( _list_builder($aref, $i) ) {
-            $seenthis{$h}++;
-        }
-        $seen{$i} = \%seenthis;
-        for (my $j = $i+1; $j <=$#{$aref}; $j++) {
-            my (%seenthat, %seenintersect);
-            my $ilabel = $i . '_' . $j;
-            $seenthat{$_}++ foreach ( _list_builder($aref, $j) );
-            foreach (keys %seenthat) {
-                $seenintersect{$_}++ if (exists $seenthis{$_});
-            }
-            $xintersection{$ilabel} = \%seenintersect;
-        }
-    }
-    return (\%seen, \%xintersection);
+    return \@seen;
 }
 
 sub _calculate_seen_only {
@@ -243,25 +203,17 @@ sub _calculate_seen_only {
     return \%seen;
 }
 
-sub _calculate_xintersection_only {
+sub _calculate_intermediate {
     my $aref = shift;
-    my (%xintersection);
-    for (my $i = 0; $i <= $#{$aref}; $i++) {
-        my %seenthis = ();
-        foreach my $h ( _list_builder($aref, $i) ) {
-            $seenthis{$h}++;
-        }
-        for (my $j = $i+1; $j <=$#{$aref}; $j++) {
-            my (%seenthat, %seenintersect);
-            my $ilabel = $i . '_' . $j;
-            $seenthat{$_}++ foreach ( _list_builder($aref, $j) );
-            foreach (keys %seenthat) {
-                $seenintersect{$_}++ if (exists $seenthis{$_});
-            }
-            $xintersection{$ilabel} = \%seenintersect;
-        }
+    my $aseenref = _calculate_array_seen_only($aref);
+    my @vals = sort { scalar(keys(%{$a})) <=> scalar(keys(%{$b})) } @{$aseenref};
+    my %intermediate = map { $_ => 1 } keys %{$vals[0]};
+    for my $l ( 1..$#vals ) {
+        %intermediate = map { $_ => 1 }
+            grep { exists $intermediate{$_} }
+            keys %{$vals[$l]};
     }
-    return \%xintersection;
+    return \%intermediate;
 }
 
 sub _calculate_union_only {
@@ -289,48 +241,62 @@ sub _calculate_union_seen_only {
     return (\%union, \%seen);
 }
 
-sub _calculate_hash_intersection {
-    my $xintersectionref = shift;
-    my @xkeys = keys %{$xintersectionref};
-    my %intersection = %{${$xintersectionref}{$xkeys[0]}};
-    for (my $m = 1; $m <= $#xkeys; $m++) {
-        my %compare = %{${$xintersectionref}{$xkeys[$m]}};
-        my %result = ();
-        foreach (keys %compare) {
-            $result{$_}++ if (exists $intersection{$_});
-        }
-        %intersection = %result;
+sub _calculate_sharedref {
+    my $seenrefsref = shift;
+
+    my %intermediate = ();
+    for my $href (@{$seenrefsref}) {
+       my %this = map { $_ => 1 } keys(%{$href});
+        for my $k (keys %this) {;
+            $intermediate{$k}++;
+        };
     }
-    return \%intersection;
+
+    my $sharedref;
+    for my $k (keys %intermediate) {
+        $sharedref->{$k}++ if $intermediate{$k} > 1;
+    }
+    return $sharedref;
 }
 
-sub _calculate_hash_shared {
-    my $xintersectionref = shift;
-    my (%shared);
-    foreach my $q (keys %{$xintersectionref}) {
-        $shared{$_}++ foreach (keys %{${$xintersectionref}{$q}});
-    }
-    return \%shared;
+sub _is_list_subset {
+    my ( $subset, $superset ) = @_;
+    # return false if the superset value is false
+    # for any subset value.
+    # note that this does *not* validate overlap of
+    # the keys; it validates the truth of supserset
+    # values.
+    $superset->{ $_ } or return 0 for keys %$subset;
+    return 1;
 }
 
 sub _subset_subengine {
     my $aref = shift;
     my (@xsubset);
-    my $seenref = _calculate_seen_only($aref);
-    my %seen = %{$seenref};
+    my %seen = %{_calculate_seen_only($aref)};
     foreach my $i (keys %seen) {
-        my %tempi = %{$seen{$i}};
         foreach my $j (keys %seen) {
-            my %tempj = %{$seen{$j}};
-            $xsubset[$i][$j] = 1;
-            foreach my $k (keys %tempi) {
-                $xsubset[$i][$j] = 0 if (! $tempj{$k});
+            if ( $i eq $j ) {
+                $xsubset[$i][$j] = 1;
+            }
+            elsif ( $i gt $j ) {
+                if ( scalar(keys %{ $seen{$i} }) == scalar(keys %{ $seen{$j} }) ){
+                    $xsubset[$i][$j] = _is_list_subset($seen{$i}, $seen{$j});
+                    $xsubset[$j][$i] = $xsubset[$i][$j];
+                }
+                elsif ( scalar(keys %{ $seen{$i} }) < scalar(keys %{ $seen{$j} }) ){
+                    $xsubset[$i][$j] = _is_list_subset($seen{$i}, $seen{$j});
+                    $xsubset[$j][$i] = 0;
+                }
+                elsif ( scalar(keys %{ $seen{$i} }) > scalar(keys %{ $seen{$j} }) ){
+                    $xsubset[$j][$i] = _is_list_subset($seen{$j}, $seen{$i});
+                    $xsubset[$i][$j] = 0;
+                }
             }
         }
     }
     return \@xsubset;
 }
-
 sub _chart_engine_regular {
     my $aref = shift;
     my @sub_or_eqv = @$aref;
@@ -376,8 +342,7 @@ sub _chart_engine_multiple {
 
 sub _equivalent_subengine {
     my $aref = shift;
-    my $xsubsetref = _subset_subengine($aref);
-    my @xsubset = @{$xsubsetref};
+    my @xsubset = @{_subset_subengine($aref)};
     my (@xequivalent);
     for (my $f = 0; $f <= $#xsubset; $f++) {
         for (my $g = 0; $g <= $#xsubset; $g++) {
@@ -394,7 +359,7 @@ sub _index_message1 {
     my $method = (caller(1))[3];
     croak "Argument to method $method must be the array index of the target list \n  in list of arrays passed as arguments to the constructor: $!"
         unless (
-                $index =~ /^\d+$/ 
+                $index =~ /^\d+$/
            and  $index <= ${$dataref}{'maxindex'}
         );
 }
@@ -413,7 +378,7 @@ sub _index_message2 {
         foreach ($index_left, $index_right) {
             croak "Each argument to method $method must be a valid array index for the target list \n  in list of arrays passed as arguments to the constructor: $!"
                 unless (
-                        $_ =~ /^\d+$/ 
+                        $_ =~ /^\d+$/
                    and  $_ <= ${$dataref}{'maxindex'}
                 );
         }
@@ -426,7 +391,7 @@ sub _index_message3 {
     my $method = (caller(1))[3];
     croak "Argument to method $method must be the array index of the target list \n  in list of arrays passed as arguments to the constructor: $!"
         unless (
-                $index =~ /^\d+$/ 
+                $index =~ /^\d+$/
            and  $index <= $maxindex
         );
 }
@@ -445,7 +410,7 @@ sub _index_message4 {
         foreach ($index_left, $index_right) {
             croak "Each argument to method $method must be a valid array index for the target list \n  in list of arrays passed as arguments to the constructor: $!"
                 unless (
-                        $_ =~ /^\d+$/ 
+                        $_ =~ /^\d+$/
                    and  $_ <= $maxindex
                 );
         }
@@ -480,7 +445,7 @@ sub _calc_seen {
         my (%seenL, %seenR);
         foreach (@$refL) { $seenL{$_}++ }
         foreach (@$refR) { $seenR{$_}++ }
-        return (\%seenL, \%seenR); 
+        return (\%seenL, \%seenR);
     } else {
         return ($refL, $refR);
     }
@@ -490,7 +455,7 @@ sub _equiv_engine {
     my ($hrefL, $hrefR) = @_;
     my (%intersection, %Lonly, %Ronly, %LorRonly);
     my $LequivalentR_status = 0;
-    
+
     foreach (keys %{$hrefL}) {
         exists ${$hrefR}{$_} ? $intersection{$_}++ : $Lonly{$_}++;
     }
@@ -547,9 +512,9 @@ sub _argument_checker_2 {
 
 # _argument_checker_3 is currently set-up to handle either 1 or 2 arguments
 # in get_unique and get_complement
-# The first argument is an arrayref holding refs to lists ('unsorted' has been 
+# The first argument is an arrayref holding refs to lists ('unsorted' has been
 # stripped off).
-# The second argument is an arrayref holding a single item (index number of 
+# The second argument is an arrayref holding a single item (index number of
 # item being tested)
 # Note:  Currently we're only checking for the quantity of arguments -- not
 # their types.  This should be fixed.
@@ -584,7 +549,7 @@ sub _argument_checker_4 {
         if (@{$args[1]} == 2) {
             my $last_index = $#{$args[0]};
             foreach my $i (@{$args[1]}) {
-		        croak "No element in index position $i in list of list references passed as first argument to function: $!"
+                croak "No element in index position $i in list of list references passed as first argument to function: $!"
                     unless ($i =~ /^\d+$/ and $i <= $last_index);
             }
             return (_argument_checker($args[0]), $args[1]);
@@ -599,9 +564,9 @@ sub _argument_checker_4 {
 sub _calc_seen1 {
     my @listrefs = @_;
     # _calc_seen1() is applied after _argument_checker(), which checks to make
-    # sure that the references in its output are either all arrayrefs 
+    # sure that the references in its output are either all arrayrefs
     # or all seenhashrefs
-    # hence, _calc_seen1 only needs to determine whether it's dealing with 
+    # hence, _calc_seen1 only needs to determine whether it's dealing with
     # arrayrefs or seenhashrefs, then, if arrayrefs, calculate seenhashes
     if (ref($listrefs[0]) eq 'ARRAY') {
         my (@seenrefs);
@@ -618,7 +583,7 @@ sub _calc_seen1 {
     }
 }
 
-# _alt_construct_tester prepares for _argument_checker in 
+# _alt_construct_tester prepares for _argument_checker in
 # get_union get_intersection get_symmetric_difference get_shared get_nonintersection
 sub _alt_construct_tester {
     my @args = @_;
@@ -631,9 +596,9 @@ sub _alt_construct_tester {
        $argref = ${$hashref}{'lists'};
        $unsorted = ${$hashref}{'unsorted'} ? 1 : '';
     } else {
-        $unsorted = shift(@args) 
+        $unsorted = shift(@args)
             if ($args[0] eq '-u' or $args[0] eq '--unsorted');
-        $argref = shift(@args); 
+        $argref = shift(@args);
     }
     return ($argref, $unsorted);
 }
@@ -654,7 +619,7 @@ sub _alt_construct_tester_1 {
         @returns = ( ${$hashref}{'lists'}, [${$hashref}{'item'}] );
         $argref = \@returns;
     } else {
-        $argref = \@args; 
+        $argref = \@args;
     }
     return $argref;
 }
@@ -678,7 +643,7 @@ sub _alt_construct_tester_2 {
 }
 
 # _alt_construct_tester_3 prepares for _argument_checker_3 in
-# get_unique get_complement 
+# get_unique get_complement
 sub _alt_construct_tester_3 {
     my @args = @_;
     my ($argref, $unsorted);
@@ -695,7 +660,7 @@ sub _alt_construct_tester_3 {
         $unsorted = ${$hashref}{'unsorted'} ? 1 : '';
     } else {
         $unsorted = shift(@args) if ($args[0] eq '-u' or $args[0] eq '--unsorted');
-        $argref = \@args; 
+        $argref = \@args;
     }
     return ($argref, $unsorted);
 }
@@ -716,7 +681,7 @@ sub _alt_construct_tester_4 {
                         : (${$hashref}{'lists'});
         $argref = \@returns;
     } else {
-        $argref = \@args; 
+        $argref = \@args;
     }
     return $argref;
 }
@@ -734,7 +699,7 @@ sub _alt_construct_tester_5 {
                     and (ref(${$hashref}{'lists'}) eq 'ARRAY') );
            $argref = ${$hashref}{'lists'};
         } else {
-           $argref = shift(@args); 
+           $argref = shift(@args);
         }
     } else {
         croak "Subroutine call requires exactly 1 reference as argument:  $!";
@@ -752,23 +717,23 @@ List::Compare::Base::_Auxiliary - Internal use only
 
 =head1 VERSION
 
-This document refers to version 0.39 of List::Compare::Base::_Auxiliary.
-This version was released July 04 2014.
+This document refers to version 0.53 of List::Compare::Base::_Auxiliary.
+This version was released June 07 2015.
 
 =head1 SYNOPSIS
 
-This module contains subroutines used within List::Compare and 
+This module contains subroutines used within List::Compare and
 List::Compare::Functional.  They are not intended to be publicly callable.
 
 =head1 AUTHOR
 
-James E. Keenan (jkeenan@cpan.org).  When sending correspondence, please 
+James E. Keenan (jkeenan@cpan.org).  When sending correspondence, please
 include 'List::Compare' or 'List-Compare' in your subject line.
 
-Creation date:  May 20, 2002.  Last modification date:  July 04 2014. 
-Copyright (c) 2002-14 James E. Keenan.  United States.  All rights reserved. 
+Creation date:  May 20, 2002.  Last modification date:  February 25 2015.
+Copyright (c) 2002-15 James E. Keenan.  United States.  All rights reserved.
 This is free software and may be distributed under the same terms as Perl
 itself.
 
-=cut 
+=cut
 
